/* 
   Copyright (C) 1992 Free Software Foundation, Inc.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
/*
   This program have been made by: Gregory VANDENBROUCK , vdb@pipo.com
   Please, send him/them a mail ( possibly with diff and/or correction )
   If you find any BUG, or if you do nay improvement */

#include "Debug.H"
// Ce ki suit, pour open()
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
// pour write
#include <unistd.h>

#include <errno.h>

#include "G_B_R_Pipe.H"

#include "MAX.H"

const int MAX_BUF = 2000;
const int MAX_TRY = 0;

G_B_R_Pipe::G_B_R_Pipe(char * name)
  :RECV(),bufsize(MAX_BUF),pos(0),pos_lecture(0),buffer(0),NomDuPipe(name)
{
  if ( (mkfifo(NomDuPipe,0644) < 0) && (errno != EEXIST) )
    { 
      Debug_Error(3,"S_Pipe::S_Pipe(mkfifo(%s))\n",NomDuPipe);
      throw RecvPipeProbs(); 
    } 

  Debug(3,"Opened pipe [%s], waiting for first call\n",NomDuPipe);

  if ((f_r = open(NomDuPipe,O_RDONLY|O_NONBLOCK)) < 0) 
    { 
      Debug_Error(3,"S_Pipe::S_Pipe(open(%s))\n",NomDuPipe); 
      throw RecvPipeProbs(); 
    }

  buffer = new char [MAX_BUF];
}


G_B_R_Pipe::~G_B_R_Pipe()
{
  Debug(0,"G_B_R_Pipe::~G_B_R_Pipe()\n");
  close(f_r);
  unlink(NomDuPipe);
  delete [] NomDuPipe;
  if ( buffer ) delete [] buffer;
}

void G_B_R_Pipe::Read(void * koa,unsigned int taille)
{
  int nb_try = 0;
  if ( ! taille ) return;
  while ( 1 )
    {
      int n = read(f_r,koa,taille);
      switch ( n )
	{
	case -1: 
	  Debug_Error(3,"C_User_C::Lit\n");
	  sleep (1);
	  if ( ++ nb_try == 3 )
	    throw RecvPipeProbs();
	  break;
	case 0:
	  throw PlusPersonne() ;
	default: return;
	}
    }
}

int G_B_R_Pipe::InitRead()
{
  unsigned char ch;
  Read(&ch,sizeof(ch));
  
  
  //printf("TYPE DE RECEPTION: %c\n",ch);
  if ( ch == '~' )
    {
      //printf("LECTURE D'UN PAQUET DE %d octets\n",SIZE_PACKET);
      if ( pos + SIZE_PACKET > bufsize )
	{
	  //printf("CHANGEMENT DE TAILLE DU BUF DE RECEPTION ! \n");
	  buffer = (char *)realloc(buffer,pos+SIZE_PACKET + 10);
	  bufsize = pos + SIZE_PACKET + 10;
	}
      Read(buffer+pos,SIZE_PACKET);
      pos += SIZE_PACKET;
      return 0;
    }
  if ( ch == '#' )
    {
      unsigned short taille;
      Read(&taille,sizeof(taille));
      //printf("Lecture d'un paquet de %d octets\n",taille);
      if ( pos + taille > bufsize )
	{
	  //printf("CHANGEMENT DE TAILLE DU BUF DE RECEPTION ! \n");
	  buffer = (char *)realloc(buffer,pos+taille + 10);
	  bufsize = pos + taille + 10;
	}
      Read(buffer+pos,taille);
      pos = 0;
      pos_lecture = 0;
      return 1;
    }
  //printf("ERREUR PROTOCOLE !\n");
  return 0;
}

void G_B_R_Pipe::Lit(void * koa,unsigned int taille)
{
  memcpy(koa,buffer+pos_lecture,taille);
  pos_lecture += taille;
}

void G_B_R_Pipe::ResetSelect(void)
{
  FD_ZERO(&fdset);
  maxfd = 0;
}

void G_B_R_Pipe::AddSelect(int a)
{
  FD_SET(a,&fdset);
  if ( a+1 > maxfd ) maxfd = a+1;
}

int G_B_R_Pipe::Select(int secondes)
{
  struct timeval Pause;

  Pause.tv_sec=secondes; Pause.tv_usec=0;

  struct timeval * p = secondes?&Pause:0;

#ifdef HP_UX
  return select(maxfd,(int *)&fdset,NULL,NULL,p);
#else
  return select(maxfd,       &fdset,NULL,NULL,p);
#endif	      
}

int G_B_R_Pipe::IsSelect(int a)
{
  return FD_ISSET(a,&fdset);
}
