/* ***************************************************************** *
 * Login                                                             *
 * ----------------------------------------------------------------- *
 * Version . . . . . . . . . . . . . . . . . . . . . . . . . . 1.1.3 *
 * ================================================================= *
 * Creation date . . . . . . . . . . . . . . . . . . Tue 01 Sep 1998 *
 * Modification date . . . . . . . . . . . . . . . . Mon 18 Jan 1999 *
 * ================================================================= *
 * Copyright (C) 1997-1998 Free Software Foundation, Inc.            *
 *                                                                   *
 * This program  is  free software;  you can  redistribute it and/or *
 * modify it  under the terms  of the GNU  General Public License as *
 * published by  the Free Software Foundation;  either version 2, or *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program  is distributed in the hope  that it will be useful, *
 * but WITHOUT  ANY WARRANTY;  without even  the implied warranty of *
 * MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along  with  this  program;  if not,  write to  the Free Software *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         *
 *                                                                   *
 * ================================================================= *
 * Authors                                                           *
 *   Sebastien APERGHIS-TRAMONI <saper@esil.univ-mrs.fr>             *
 *                                                                   *
 * If you find any bug, or if you do any improvment, please send us  *
 * a mail (possibly with diff and/or correction).                    *
 *                                                                   *
 * ***************************************************************** */


#ifndef LOGIN_H
#define LOGIN_H


/* ***************************************************************** *
 * defines                                                           *
 * ***************************************************************** */
#define LOGIN_MAX_IDLE      5*60    // Max idle time
#define MAX_TRIES           5       // Max tries
#define PASSWORD_MIN_LENGTH 4       // Password minimum length
#define SLEEP_TIME          10      // Sleep time at exit


/* ***************************************************************** *
 * classes                                                           *
 * ***************************************************************** */
class Login {
  public: 
    // Constructor and destructor.
    Login               (void);
    ~Login              (void);
    
    // Menus. 
    int     Logon           (void);
    void    About           (void);
    void    Help            (void);
    void    NewUser         (void);
    void    Clean           (void);
    void    Nickname        (void);
    void    Passwd          (void);
    void    Email           (void);
    void    ChooseLang      (void);
    void    RealName        (void);
    void    ConfirmInfos    (void);
    
    // Utility functions.
    void    Launch          (void);
    int     WriteData       (void);
    int     UserNumber      (void);
    int     GetName         (char * str, int len);
    char *  GetPassword     (char * prompt);
    int     GetString       (char * str, int len);
    time_t  GetTime         (void);
    int     GetKey          (void);
    char *  GetHostname     (void);
    int     Taboo           (char * name);
    int     CheckEmailAddress       (char * email);
    int     SendEmail               (char * address, char * mail);
    int     SendValidationMail      (void);
    int     SendPasswordFailureMail (void);
    
    // Properties.
    char *  login           ;
    int     usernum         ;
    char *  passwd          ;
    char *  email           ;
    int     language        ;
    int     key             ;
    
    // Optional properties. 
    char *  name            ;
};


class UserCancelled {};

class CreationError {};

#endif /* LOGIN_H */
