/* ***************************************************************** *
 * Atlantis BBS - Profile CGI                                        *
 * ----------------------------------------------------------------- *
 * Version . . . . . . . . . . . . . . . . . . . . . . . . .  1.0.12 *
 * ================================================================= *
 * Creation date . . . . . . . . . . . . . . . . . . Wed 29 Jul 1998 *
 * Modification date . . . . . . . . . . . . . . . . Tue 05 Jan 1999 *
 * ================================================================= *
 * Copyright (C) 1997-1998 Free Software Foundation, Inc.            *
 *                                                                   *
 * This program  is  free software;  you can  redistribute it and/or *
 * modify it  under the terms  of the GNU  General Public License as *
 * published by  the Free Software Foundation;  either version 2, or *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program  is distributed in the hope  that it will be useful, *
 * but WITHOUT  ANY WARRANTY;  without even  the implied warranty of *
 * MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along  with  this  program;  if not,  write to  the Free Software *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         *
 *                                                                   *
 * ================================================================= *
 * Authors                                                           *
 *   Sebastien APERGHIS-TRAMONI <saper@esil.univ-mrs.fr>             *
 *   Gregory VANDENBROUCK <vdb@pipo.com>                             *
 *                                                                   *
 * If you find any bug, or if you do any improvment, please send us  *
 * a mail (possibly with diff and/or correction).                    *
 *                                                                   *
 * ***************************************************************** */


/* ***************************************************************** *
 * standart includes                                                 *
 * ***************************************************************** */
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <ctype.h>
#include <errno.h>
#include <time.h>
#include <fcntl.h>


/* ***************************************************************** *
 * Unix includes                                                     *
 * ***************************************************************** */
#include <sys/types.h>
#include <sys/stat.h>


/* ***************************************************************** *
 * Pipo-BBS includes                                                 *
 * ***************************************************************** */
#include "Con_Wholist.H"
#include "G_Path.H"
#include "G_User.H"
#include "consts.H"


/* ***************************************************************** *
 * CGI includes                                                      *
 * ***************************************************************** */
#include "CGI_parser.h"
#include "CGI_colors.h"
#include "CGI_HTML.h"
#include "log_error.h"


/* ***************************************************************** *
 * defines                                                           *
 * ***************************************************************** */
#define BUF_SIZE    512
#define PATH        "/bbs/src/www/html/"


/* ***************************************************************** *
 * prototypes                                                        *
 * ***************************************************************** */
char *  Language    (int language);
void    View        (void);
void    terminate   (void);

 
/* ***************************************************************** *
 * global variables                                                  *
 * ***************************************************************** */
static char     program[]           = "Profile Gateway";
static char     version[]           = "1.0.12";

char            Form_name[]         = "login";
char            Lang[]              = "lang";
char            Profile[]           = "profile";

char            Hidden_Field[]      = "<FONT Color=\"#FF0000\">* hidden *</FONT>";

char            NoData_err_fr[]     = PATH "NoData.fr.html";
char            NoSuchUser_err_fr[] = PATH "NoSuchUser.fr.html";
char            BadUser_err_fr[]    = PATH "BadUser.fr.html";

G_Path          G(60)               ; // maximum idle time: 60 sec
Con_Wholist     W                   ;
G_User *        user                ;



/* ***************************************************************** *
 * Canonize()                                                        *
 * --------                                                          *
 * description                                                       *
 *   This function canonizes the given name:                         *
 *    - only alphabetic characters are allowed                       *
 *    - no multiple spaces                                           *
 *    - the letter following a space must be capital                 *
 *                                                                   *
 * ***************************************************************** */
void Canonize(char * name) {
    char    str[LG_NOM_USER + 1];
    char *  s = NULL;
    int     must_upper = 1;
    unsigned    i = 0;
    unsigned    k = 0;
    
    strcpy(str, name);
    
    for(i=0; i<strlen(str); i++) {
        
        // We remove non-alphabetic characters.
        if(str[i] != ' ' && !isalpha(str[i])) {
            for(k=i; k<strlen(str); k++) {
                str[k] = str[k+1];
            }
        }
        
        if(isupper(str[i])) {
            must_upper = 0;
        }
        
        if(str[i] == ' ') {
            // Space must be followed by a capital letter.
            must_upper = 1;
            continue;
        }
        
        if(must_upper && islower(str[i])) {
            str[i] = toupper(str[i]);
            must_upper = 0;
        }
    }
    
    // Now we eliminate multiple spaces. 
    // The trick is that strtok() collapse multiples spaces. 
    
    s = strtok(str, " ");
    strcpy(name, s);
    
    while((s = strtok(NULL, " ")) != NULL) {
        strcat(name, " ");
        strcat(name, s);
    }
}


/* ***************************************************************** *
 * Language()                                                        *
 *                                                                   *
 * ***************************************************************** */
char * Language(int language) {
    switch(language) {
        case L_fr: return "French";    break;
        case L_uk: return "English";   break;
        case L_sp: return "Spanish";   break;
        case L_it: return "Italian";   break;
        case L_no: return "Norvegian"; break;
        ///case L_de: return "German";    break;
        default:   return "";          break;
    }
}


/* ***************************************************************** *
 * View()                                                            *
 *                                                                   *
 * ***************************************************************** */
void View(void) {
    FILE *  file        = NULL;
    char *  filename    = NULL;
    char *  str         = NULL;
    char *  html        = NULL;
    time_t  now         = time(NULL);
    time_t  log         = 0;
    int     monthes     = 0;
    int     days        = 0;
    int     hours       = 0;
    int     minutes     = 0;
    int     secondes    = 0;
    int     hidden_fields = user->D->Num(N_cflag);
    int     n           = 0;
    
    // Sending HTML document
    Standard_HTML_Top("Profile Gateway");
    
    printf("<TABLE Border=1 CellSpacing=0 Align=Center>\n<TR>\n<TD>\n"
           "<TABLE Border=0 CellSpacing=0 Width=100%%>\n<TR>\n"
           "<TD VAlign=Top BGColor=\"#EEEEEF\">\n\n"
           "<TABLE Border=0 CellSpacing=0 Width=100%% BGColor=\"#EEEEEF\">\n"
           "<TR><TD BGColor=\"#6699EE\" ColSpan=2><B><I>&nbsp;"
           "Information about a user.</I></B><BR></TD></TR>\n"
    );
    
    // Nickname --
    printf("<TR><TD BGColor=\"#D0D0D1\">&nbsp;&nbsp;&nbsp;"
           "<Big><I>%s</I></Big></TD>\n", 
           (user->Nick.null()) ? "*No nickname (?)*" : user->Nick.data()
    );
    
    // Extra --
    str = (char *)user->D->St(C_extra).data();
    html = bbs2html(str);
    printf("<TD BGColor=\"#D0D0D1\">&nbsp;&nbsp;&nbsp;%s<BR></TD>\n", (html) ? html : "");
    delete html;
    
    // User Number --
    if(hidden_fields & U_C_num) {
        printf("<TR><TD><B>Number :</B></TD>         <TD>%s<BR></TD></TR>\n", Hidden_Field);
    } else {
        printf("<TR><TD><B>Number :</B></TD>         <TD>%ld<BR></TD></TR>\n", 
               user->D->Num(N_numdeuser)
        );
    }
    
    // Real Name --
    if(hidden_fields & U_C_name) {
        printf("<TR><TD><B>Name :</B></TD>    <TD>%s<BR></TD></TR>\n", Hidden_Field);
    } else {
        if(!user->D->St(C_nom).null()) {
            printf("<TR><TD><B>Name :</B></TD>    <TD>%s<BR></TD></TR>\n", 
                   user->D->St(C_nom).data()
            );
        }
    }
    
    // Stripes --
    if(user->D->Num(N_flags)) {
        printf("<TR><TD><B>Stripes :</B></TD> <TD>");
        n = user->D->Num(N_flags);
        if(n & D_admin) printf("<FONT Color=\"#DD0000\">Armateur</FONT> " );
        if(n & D_kick ) printf("<FONT Color=\"#EE0066\">Capitaine</FONT> ");
        if(n & D_users) printf("<FONT Color=\"#FFB00B\">Amiral</FONT> "   );
        if(n & D_rooms) printf("<FONT Color=\"#6666CC\">Timonier</FONT> " );
        if(n & D_verif) printf("<FONT Color=\"#CC5566\">Vigie</FONT> "    );
        if(n & D_help ) printf("<FONT Color=\"#70BB80\">Barreur</FONT> "  );
        printf("<BR></TD></TR>\n");
    }
    
    // On line time and hostname --
    if(W.connected(user->numdeuser)) {
        // Currently online
        log = now - user->D->Num(N_lastlog);
        printf("<TR><TD ColSpan=2><B>Connected</B> since ");
        
    } else {
        // Not online
        log = now - user->D->Num(N_logout);
        printf("<TR><TD ColSpan=2>Last connected : ");
    }
    
    str = (char *)user->D->St(C_last).data(); // Last host
    
    secondes =  log            % 60;
    minutes  = (log / 60     ) % 60;
    hours    = (log / 3600   ) % 24;
    days     = (log / 86400  ) % 30;
    monthes  =  log / 2592000;
    
    if(monthes > 0) printf("%d month%s, ", monthes, (monthes > 1) ? "es" : "");
    if(days    > 0) printf("%d day%s, ", days, (days > 1) ? "s" : "");
    if(hours   > 0) printf("%d hour%s and ", hours, (hours > 1) ? "s" : "");
    printf("%d minute%c from %s.<BR></TD></TR>\n", 
           minutes, (minutes > 1) ? 's' : ' ', 
           (str) ? str : "we don't know where"
    );
    
    // War cry --
    if(!user->D->St(C_doing).null()) {
        str = (char *)user->D->St(C_doing).data();
        html = bbs2html(str);
        printf("<TR><TD BGColor=\"#D0D0D1\"><B>War cry :</B></TD>        "
               "<TD BGColor=\"#D0D0D1\">%s<BR></TD></TR>\n", html
        );
        delete html;
    }
    
    // Email address --
    if(hidden_fields & U_C_mail) {
        printf("<TR><TD><B>Mail :</B></TD>           <TD>%s<BR></TD></TR>\n", Hidden_Field);
        
    } else {
        str = (char *)user->D->St(C_mail).data();
        if(!user->D->St(C_mail).null()) {
            printf("<TR><TD><B>Mail :</B></TD>           "
                   "<TD><A HREF=\"mailto:%s\">%s</A><BR></TD></TR>\n", str, str
            );
        }
    }
    
    // Home page --
    if(!user->D->St(C_url).null()) {
        str = (char *)user->D->St(C_url).data();
        printf("<TR><TD><B>Web page :</B></TD>       "
               "<TD><A HREF=\"%s\" Target=\"_new\">%s%s</A><BR></TD></TR>\n", 
               str, (strstr(str, "http://") == NULL) ? "http://" : "" , str
        );
    }
    
    // Curriculum Vitae --
    if(!user->D->St(C_CV).null()) {
        str = (char *)user->D->St(C_CV).data();
        printf("<TR><TD><B>Curriculum Vit&aelig; :</B></TD>     "
               "<TD><A HREF=\"%s\" Target=\"_new\">%s%s</A><BR></TD></TR>\n", 
               str, (strstr(str, "http://") == NULL) ? "http://" : "" , str
        );
    }
    
    // ICQ Number --
    if(user->D->Num(N_ICQ)) {
        printf("<TR><TD><B>ICQ :</B></TD>            <TD>%ld<BR></TD></TR>\n", 
               user->D->Num(N_ICQ)
        );
    }
    
    // Tag Score --
    if(user->PlayCoco()) {
        printf("<TR><TD><B>Coco score :</B></TD>     <TD>%ld<BR></TD></TR>\n", 
               user->D->Num(N_TagSc)
        );
    }
    
    
    // Snail address --
    if(hidden_fields & U_C_pays) {
        printf("<TR><TD><B>Snail address :</B></TD> <TD>%s<BR></TD></TR>\n", Hidden_Field);
        
    } else {
        if( !user->D->St(C_adresse).null() || !user->D->St(C_zip).null() || 
            !user->D->St(C_ville).null()   || !user->D->St(C_pays).null()   ) {
            printf("<TR><TD><B>Snail address :</B></TD> "
                   "<TD>%s%c %s %s %c %s<BR></TD></TR>\n", 
                   user->D->St(C_adresse).data(), 
                   (!user->D->St(C_adresse).null()) ? ',' : ' ', 
                   user->D->St(C_zip).data(), 
                   user->D->St(C_ville).data(),
                   (!user->D->St(C_ville).null()) ? '-' : ' ',
                   user->D->St(C_pays).data()
            );
        }
    }
    
    // Phone --
    if(0) {  //if(hidden_fields & U_C_tel) {
        printf("<TR><TD><B>Phone :</B></TD>     <TD>%s<BR></TD></TR>\n", Hidden_Field);
        
    } else {
        if(!user->D->St(C_tel).null()) {
            printf("<TR><TD><B>Phone :</B></TD>     <TD>%s<BR></TD></TR>\n", 
                   user->D->St(C_tel).data()
            );
        }
    }
    
    // Mother language --
    printf("<TR><TD><B>Native language :</B></TD>             "
           "<TD>%s&nbsp;<BR></TD></TR>\n", 
           Language(user->D->Num(N_langue))
    );
    
    // Spoken languages --
    printf("<TR><TD><B>Spoken languages :</B></TD>            <TD>");
    n = user->D->Num(N_comprises);
    if(n & L_fr) printf("%s ", Language(L_fr));
    if(n & L_uk) printf("%s ", Language(L_uk));
    if(n & L_sp) printf("%s ", Language(L_sp));
    if(n & L_it) printf("%s ", Language(L_it));
    if(n & L_no) printf("%s ", Language(L_no));
    //if(n & L_de) printf("%s ", Language(L_de));
    printf("&nbsp;<BR></TD></TR>\n");
    
    // Miscellaneous statistics --
    printf("<TR><TD><B>Statistics :</B></TD>    <TD>");
    
    // -- Number of logins
    if(hidden_fields & U_C_log) {
        printf("%s, ", Hidden_Field);
    } else {
        printf("%ld embarkings, ", user->D->Num(N_logins));
    }
    
    // -- Total online time
    if(hidden_fields & U_C_time) {
        printf("%s, ", Hidden_Field);
    } else {
        printf("%ld hours, ", user->D->Num(N_tpspasse) / 3600);
    }
    
    // -- Percent of life spent on board
    printf("%ld%% of his/her life of sailor.<BR></TD></TR>\n", 
           (user->D->Num(N_tpspasse) * 100) / (now - user->D->Num(N_first))
    );
    
    // -- Number of messages sent and received; number of posts
    printf("<TR><TD>&nbsp;</TD> "
           "<TD>%ld messages sent, %ld messages received, %ld posts<BR></TD></TR>\n", 
           user->D->Num(N_Xe), 
           user->D->Num(N_Xr), 
           user->D->Num(N_Post) 
    );
    
    printf("</TABLE>\n</TD>\n\n");
    
    // Photo --
    str = user->Nick.data();
    n = strlen(str);
    str = strcpy(new char [n], str);
    space2plus(str);
    printf("<TD VAlign=Top><IMG Src=\"/cgi-bin/get_image?%s=%s\" "
           "Alt=\"[Photo]\"><BR></TD>\n", 
           Form_name, str
    );
    
    printf("</TR>\n\n");
    
    // Custom Profile --
    filename = G.GetUserFile(user->Nick, Profile);
    printf("<TR><TD ColSpan=2 BGColor=\"#333333\"> <!-- Custom Profile -->\n");
    
    if(filename != NULL && strlen(filename) > 1) {
        
        file = fopen(filename, "r");
        
        if(file != NULL) {
            
            printf("<PRE><FONT Color=\"#FFFFFF\" Size=3>");
            
            // str[] will be used as a buffer.
            str = new char[BUF_SIZE + 1];

            if(str == NULL) {
                fprintf(stderr, "%s: new[] failed; can't allocate buffer\n", program);
            }
            
            while((n = fread(str, sizeof(char), BUF_SIZE, file)) != 0) {
                str[n] = 0;
                html = bbs2html(str);
                fwrite(html, sizeof(char), strlen(html), stdout);
                delete html;
            }
            
            delete str;
            
            fclose(file);
        }
    }
    
    printf("</FONT></PRE>\n</TD></TR>\n"
           "</TABLE>\n\n"
           "</TD></TR>\n"
           "</TABLE>\n\n"
           "<BR>\n<HR>\n"
           "Note that if you have an account, you can add your own photo by connecting\n"
           "to the ftp server bbs.resus.org and putting it in the /pub/incoming directory.\n"
    );
    
    Standard_HTML_Bottom(program, version);
}


/* ***************************************************************** *
 * main()                                                            *
 *                                                                   *
 * ***************************************************************** */
int main(int argc, char ** argv) {
    HASH *  data;
    char *  name;
    
    open_log_file();
    setbuf(stdout, NULL);  // Unbuffer stdout.
    setbuf(stderr, NULL);  // Unbuffer stderr.
    
    // We want a name. 
    // If there's at least one argument (argc > 1), that means
    // this program has been launched under a shell, and the name
    // should be given by the first argument; otherwise, we assume
    // that the program has been launched as a CGI, so the name 
    // should be pass by one of the standard method (GET or POST). 
    
    if(argc > 1) {
        // Shell launch: we use the first argument. 
        name = argv[1];
        printf("%s, version %s\n", program, version);
        
    } else {
        // CGI launch: we use the standard CGI method. 
        data = expect(5);
        parse(data);
        name = get_value(data, Form_name);
    }
    
    // Sending HTTP Header
    Standard_HTTP_Header();
    
    if(name == NULL || strlen(name) <= 1) {
        print_file(NoData_err_fr, stdout);
        purge(data);
        return 1;
    }
    
    Canonize(name);
    
    // If we are here, we should have a valid name, therefore
    // we can try to find the user data. 
    
    try {
        user = new G_User(name);
    }
    
    catch(BadUser) {
        print_file(BadUser_err_fr, stdout);
        purge(data);
        delete user;
        return 1;
    }
    
    // One last check..
    if(!user) {
        print_file(NoSuchUser_err_fr, stdout);
    }
    
    // If we are here, the user data is valid, so we can print
    // out the informations. 
    
    View();
    delete user;
    purge(data);
    
    return 0;
}


/* ***************************************************************** *
 * terminate()                                                       *
 *                                                                   *
 * ***************************************************************** */
void terminate(void) {
    fprintf(stderr, "%s: terminate()\n", program);
    exit(1);
}

