/* ***************************************************************** *
 * Atlantis BBS - WhoList CGI                                        *
 * ----------------------------------------------------------------- *
 * Version . . . . . . . . . . . . . . . . . . . . . . . . . . 1.0.8 *
 * ================================================================= *
 * Creation date . . . . . . . . . . . . . . . . . . Wed 29 Jul 1998 *
 * Modification date . . . . . . . . . . . . . . . . Fri 01 Jan 1999 *
 * ================================================================= *
 * Copyright (C) 1997-1998 Free Software Foundation, Inc.            *
 *                                                                   *
 * This program  is  free software;  you can  redistribute it and/or *
 * modify it  under the terms  of the GNU  General Public License as *
 * published by  the Free Software Foundation;  either version 2, or *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program  is distributed in the hope  that it will be useful, *
 * but WITHOUT  ANY WARRANTY;  without even  the implied warranty of *
 * MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along  with  this  program;  if not,  write to  the Free Software *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         *
 *                                                                   *
 * ================================================================= *
 * Authors                                                           *
 *   Sebastien APERGHIS-TRAMONI <saper@esil.univ-mrs.fr>             *
 *   Cyril GIRON <cgiron@writeme.com>                                *
 *   Gregory VANDENBROUCK <vdb@pipo.com>                             *
 *                                                                   *
 * If you find any bug, or if you do any improvment, please send us  *
 * a mail (possibly with diff and/or correction).                    *
 *                                                                   *
 * ***************************************************************** */


/* ***************************************************************** *
 * standart includes                                                 *
 * ***************************************************************** */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <time.h>


/* ***************************************************************** *
 * Pipo-BBS includes                                                 *
 * ***************************************************************** */
#include "Con_Wholist.H"
#include "G_Path.H"
#include "consts.H"


/* ***************************************************************** *
 * CGI includes                                                      *
 * ***************************************************************** */
#include "CGI_colors.h"
#include "CGI_parser.h"
#include "CGI_HTML.h"
#include "log_error.h"


/* ***************************************************************** *
 * global variables                                                  *
 * ***************************************************************** */
static char program[]           = "Wholist CGI";
static char version[]           = "1.0.8";

static char Language[]          = "lang";
static char Default_Language[]  = "en";
static char Refresh[]           = "refresh";
static int  Refresh_delay       = 20;

G_Path      G(60)               ; // one minute max
Con_Wholist W                   ;



/* ***************************************************************** *
 * main()                                                            *
 *                                                                   *
 * ***************************************************************** */
int main(void) {
    HASH *      data    = NULL;
    time_t      now     = time(NULL);
    struct tm * time    = localtime(&now);
    USRINFO *   user    = NULL;
    char *      lang    = NULL;
    char *      name    = NULL;
    char *      doing   = NULL;
    int         flags   = 0;
    int         minutes = 0;
    int         hours   = 0;
    int         i       = 0;
    
    open_log_file();
    setbuf(stdout, NULL);  // Unbuffer stdout.
    setbuf(stderr, NULL);  // Unbuffer stderr.
    
    data = expect(5);
    parse(data);
    lang = get_value(data, Language);
    
    if(lang == NULL) {
        lang = Default_Language;
    }
    
    i = (get_value(data, Refresh) != NULL) ? atoi(get_value(data, Refresh)) : 0;
    Refresh_delay = (i > Refresh_delay) ? i : Refresh_delay;
    
    // Sending HTTP Header
    Standard_HTTP_Header();
    
    // Sending HTML document
    printf("<HTML>\n<HEAD>\n"
           "<TITLE>Atlantis BBS - WhoList</TITLE>\n"
           "<META HTTP-EQUIV=\"Refresh\" CONTENT=%d>\n"
           "</HEAD>\n"
           "<BODY BGCOLOR=\"#FFFFFF\" Text=\"#000000\" "
           "Link=\"#00FF00\" VLink=\"#00FF00\">\n", 
           Refresh_delay
    );
    printf("<H1 Align=Center>Atlantis BBS - WhoList</H1>\n"
           "<Center>Refreshed every %d seconds.</Center><BR>\n"
           "<TABLE Border=0 Align=Center CellSpacing=0 BGColor=\"#000000\">"
           "<TR>\n<TH ColSpan=6 Align=Center>", 
           Refresh_delay
    );
    printf("<FONT Color=\"#00FFFF\">There %s</FONT> "
           "<FONT Color=\"#FFFF00\">%d</FONT> "
           "<FONT Color=\"#00FFFF\">sailor%s on board at "
           "<FONT Color=\"#FFFF00\">%d</FONT>"
           "<FONT Color=\"#FFFFFF\">:</FONT>"
           "<FONT Color=\"#FFFF00\">%02d</FONT>"
           "<FONT Color=\"#00FFFF\">, local time</FONT><BR></TH>\n</TR>\n",
           (W.NB() > 1) ? "are" : "is", 
           W.NB(),
           (W.NB() > 1) ? "s" : "", 
           time->tm_hour,
           time->tm_min
    );
    printf("<TR BGColor=\"#0000FF\">\n"
           "<TD><FONT Color=\"#FFFFFF\">User</FONT>&nbsp;</TD>\n"
           "<TD><FONT Color=\"#FFFFFF\">Stripes</FONT>&nbsp;</TD>\n"
           "<TD><FONT Color=\"#FFFFFF\">On board</FONT>&nbsp;</TD>\n"
           "<TD><FONT Color=\"#FFFFFF\">Idle</FONT>&nbsp;</TD>\n"
           "<TD><FONT Color=\"#FFFFFF\">Hostname</FONT>&nbsp;</TD>\n"
           "<TD><FONT Color=\"#FFFFFF\">War cry</FONT>&nbsp;<BR></TD>\n</TR>\n"
    );
    
    for(i=0; i<W.NB(); i++) {
        user = W.User(W.mem->ki[i]);
        
        if(!user)
            continue;
        
        printf("<TR>\n");
        
        // User Name --
        name = W.name(W.mem->ki[i]);
        space2plus(name);
        printf("<TD><A HREF=\"/cgi-bin/Profile?login=%s&lang=%s\">", name, lang);
        plus2space(name);
        printf("<FONT Color=\"#00FF00\">%s</FONT></A>&nbsp;</TD>\n", name);
        
        // User Flags --
        flags = user->u_wholist;
        printf("<TD Align=Center><FONT Color=\"#FFFF00\">");
        if(flags & W_Hidden) printf("!");   // Hidden
        if(flags & W_Tag   ) printf("C");   // Coco
        if(flags & W_Help  ) printf("%%");  // SysGuide
        if(flags & W_Disabl) printf("*");   // X-Disabled
        if(flags & W_poste ) printf("+");   // Posting
        if(flags & W_X     ) printf("x");   // Xing
        if(flags & W_buisy ) printf("?");   // Busy
        if(flags & W_Chat  ) printf("&");   // Chatting
        if(!(user->u_options & U_Verified)) printf("V");   // Unvalidated
        printf("</FONT>&nbsp;</TD>\n");
        
        // Connected Time --
        printf("<TD Align=Right> <FONT Color=\"#00FFFF\">");
        minutes = (now - user->login) / 60;
        hours = minutes / 60;
        minutes %= 60;
        printf("%d:%02d</FONT>&nbsp;</TD>\n", hours, minutes);
        
        // Idle Time --
        printf("<TD Align=Right> <FONT Color=\"#FF00FF\">");
        minutes = (now - user->last) / 60;
        hours = minutes / 60;
        minutes %= 60;
        printf("%d:%02d</FONT>&nbsp;</TD>\n", hours, minutes);
        
        // Hostname --
        printf("<TD> <FONT Color=\"#FF0000\">%s</FONT>&nbsp;</TD>\n", user->From);
        
        // Doing --
        if(flags & W_Disabl) {
            // X-Disabled, so we replace the normal war cry 
            // with the disable reason. 
            doing = bbs2html(user->Raison);
            printf("<TD> <FONT Color=\"#00FFFF\">%s%s</FONT><BR></TD>\n", doing, reset);
            delete doing;
            
        } else {
            doing = bbs2html(user->Doing);
            printf("<TD> <FONT Color=\"#00FFFF\">%s%s</FONT><BR></TD>\n", doing, reset);
            delete doing;
        }
        
        printf("</TR>\n");
    }
    
    printf("</TABLE>\n<BR>\n");
    //print_file("/bbs/html/wholist.stripes.html", stdout);
    
    Standard_HTML_Bottom(program, version);
    
    return 0;
}


/* ***************************************************************** *
 * terminate()                                                       *
 *                                                                   *
 * ***************************************************************** */
void terminate(void) {
  fprintf(stderr, "%s: terminate()\n", program);
  exit(1);
}

