/* ***************************************************************** *
 * Atlantis BBS - get_image                                          *
 * ----------------------------------------------------------------- *
 * Version . . . . . . . . . . . . . . . . . . . . . . . . . . 1.0.1 *
 * ================================================================= *
 * Creation date . . . . . . . . . . . . . . . . . . Fri 28 Aug 1998 *
 * Modification date . . . . . . . . . . . . . . . . Wed 02 Dec 1998 *
 * ================================================================= *
 * Copyright (C) 1997-1998 Free Software Foundation, Inc.            *
 *                                                                   *
 * This program  is  free software;  you can  redistribute it and/or *
 * modify it  under the terms  of the GNU  General Public License as *
 * published by  the Free Software Foundation;  either version 2, or *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program  is distributed in the hope  that it will be useful, *
 * but WITHOUT  ANY WARRANTY;  without even  the implied warranty of *
 * MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along  with  this  program;  if not,  write to  the Free Software *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         *
 *                                                                   *
 * ================================================================= *
 * Authors                                                           *
 *   Sebastien APERGHIS-TRAMONI <saper@esil.univ-mrs.fr>             *
 *                                                                   *
 * If you find any bug, or if you do any improvment, please send us  *
 * a mail (possibly with diff and/or correction).                    *
 *                                                                   *
 * ***************************************************************** */


/* ***************************************************************** *
 * standart includes                                                 *
 * ***************************************************************** */
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <ctype.h>
#include <errno.h>
#include <time.h>
#include <fcntl.h>


/* ***************************************************************** *
 * Unix includes                                                     *
 * ***************************************************************** */
#include <sys/types.h>
#include <sys/stat.h>


/* ***************************************************************** *
 * Pipo-BBS includes                                                 *
 * ***************************************************************** */
#include "G_Path.H"
#include "String.H"


/* ***************************************************************** *
 * CGI includes                                                      *
 * ***************************************************************** */
#include "CGI_parser.h"
#include "log_error.h"


/* ***************************************************************** *
 * defines                                                           *
 * ***************************************************************** */
#define BUF_SIZE    1024
#define valid(c)    ( ( ((c) >= 'a') && ((c) <= 'z') ) || \
                      ( ((c) >= 'A') && ((c) <= 'Z') ) || \
                      ( (c) == ' ' )                      \
                    )


/* ***************************************************************** *
 * prototypes                                                        *
 * ***************************************************************** */
int     trustable   (char * filename);
int     send_file   (char * filename);
void    terminate   (void);

 
/* ***************************************************************** *
 * global variables                                                  *
 * ***************************************************************** */
static char     program[]           = "get_image";
static char     version[]           = "1.0.1";
static char     crlf[]              = "\015\012";   // Network end-of-line

static char     Login[]             = "login";
static char     Default_Image[]     = "/bbs/src/www/data/Belamynette.jpg";
static char     Photo_gif[]         = "Photo.gif";
static char     Photo_jpg[]         = "Photo.jpg";
static char     type_Gif[]          = "gif";
static char     type_Jpeg[]         = "jpeg";

G_Path          G(60)               ; // Max idle time: 1 min



/* ***************************************************************** *
 * trustable()                                                       *
 *                                                                   *
 * ***************************************************************** */
int trustable(char * filename) {
    char *  s = filename;
    
    while(*s != 0) {
        if(valid(*s)) {
            s++;
        } else {
            return 0;
        }
    }
    
    return 1;
}


/* ***************************************************************** *
 * send_file()                                                       *
 *                                                                   *
 * ***************************************************************** */
int send_file(char * filename) {
    FILE *  file = NULL;
    char    buf[BUF_SIZE];
    int     n = 0;
    
    file = fopen(filename, "r");
    
    if(file == NULL) {
        fprintf(stderr, 
                "%s: in send_file(): can't open '%s': error %d: %s\n", 
                program, filename, errno, strerror(errno)
        );
        return 0;
    }
    
    setbuf(file, NULL);
    
    while((n = fread(buf, sizeof(char), BUF_SIZE, file)) != 0) {
        fwrite(buf, sizeof(char), n, stdout);
    }
    
    return 1;
}


/* ***************************************************************** *
 * main()                                                            *
 *                                                                   *
 * ***************************************************************** */
int main(int argc, char ** argv) {
    String  user;
    struct stat info;
    HASH *  data = NULL;
    char *  name = NULL;
    char *  filename;
    char *  type = NULL;
    int     size = 0;
    
    open_log_file();
    setbuf(stdout, NULL);  // Unbuffer stdout.
    setbuf(stderr, NULL);  // Unbuffer stderr.
    
    // We want a name. 
    // If there's at least one argument (argc > 1), that means
    // this program has been launched under a shell, and the name
    // should be given by the first argument; otherwise, we assume
    // that the program has been launched as a CGI, so the name 
    // should be pass by one of the standard method (GET or POST). 
    
    // Note
    // ----
    // When I wrote the comment below, I didn't test this program 
    // very much; it worked, and I didn't verify the correctness of
    // my programming. This is why it took me time to track the bug
    // that prevent this program to send the image of logins containing
    // one or more spaces. 
    // This bug came from the fact that when you get an url like 
    // 'http://server/cgi-bin/program?argument' the server considers 
    // this like an ISINDEX query, and the argument is given throught 
    // argv[] to the program. Therefore when the argument contains 
    // spaces, 'http://server/cgi-bin/program?foo+bar', the program
    // receives argv[] = { "program", "foo", "bar" } ... where one 
    // (like me) could expect argv[] = { "program", "foo bar" }
    
    if(argc > 1) {
        // Shell launch: we use the first argument. 
        name = argv[1];
        
    } else {
        // CGI launch: we use the standard CGI method. 
        data = expect(1);
        parse(data);
        name = get_value(data, Login);
    }
    
    if(name == NULL || strlen(name) < 1) {
        purge(data);
        return 1;
    }
    
    user = String(name);
    
    switch(trustable(name)) {
        default: 
            // We first test if the image is a GIF.
            filename = G.GetUserFile(user, Photo_gif);
            if(filename != NULL && stat(filename, &info) == 0) {
                type = type_Gif;
                break;
            }
            
            // Now we test if it's a Jpeg. 
            filename = G.GetUserFile(user, Photo_jpg);
            if(filename != NULL && stat(filename, &info) == 0) {
                type = type_Jpeg;
                break;
            }
            
            // If we are here, that means the image is not of
            // a known format, or that the user has no custom
            // photo. 
            filename = Default_Image;
            type = type_Jpeg;
    }
    
    stat(filename, &info);
    size = info.st_size;
    
    // Sending HTTP Header
    printf("Status: 200 OK%s", crlf);
    printf("Content-Type: image/%s%s", type, crlf);
    printf("Content-Length: %d%s%s", size, crlf, crlf);
    
    send_file(filename);
    
    purge(data);
    
    return 0;
}


/* ***************************************************************** *
 * terminate()                                                       *
 *                                                                   *
 * ***************************************************************** */
void terminate(void) {
    fprintf(stderr, "%s: terminate()\n", program);
    exit(1);
}

